import { TableMetadata } from "../metadata/TableMetadata";
import { RelationMetadata } from "../metadata/RelationMetadata";
import { IndexMetadata } from "../metadata/IndexMetadata";
import { CompoundIndexMetadata } from "../metadata/CompoundIndexMetadata";
import { ColumnMetadata } from "../metadata/ColumnMetadata";
import { EventSubscriberMetadata } from "../metadata/EventSubscriberMetadata";
import { EntityListenerMetadata } from "../metadata/EntityListenerMetadata";
import { NamingStrategyMetadata } from "../metadata/NamingStrategyMetadata";
import { JoinColumnMetadata } from "../metadata/JoinColumnMetadata";
import { JoinTableMetadata } from "../metadata/JoinTableMetadata";
import { TargetMetadataCollection } from "../metadata/collection/TargetMetadataCollection";
import { PropertyMetadataCollection } from "../metadata/collection/PropertyMetadataCollection";
/**
 * Storage all metadatas of all available types: tables, fields, subscribers, relations, etc.
 * Each metadata represents some specifications of what it represents.
 */
export declare class MetadataStorage {
    readonly tableMetadatas: TargetMetadataCollection<TableMetadata>;
    readonly namingStrategyMetadatas: TargetMetadataCollection<NamingStrategyMetadata>;
    readonly eventSubscriberMetadatas: TargetMetadataCollection<EventSubscriberMetadata>;
    readonly compoundIndexMetadatas: TargetMetadataCollection<CompoundIndexMetadata>;
    readonly columnMetadatas: PropertyMetadataCollection<ColumnMetadata>;
    readonly relationMetadatas: PropertyMetadataCollection<RelationMetadata>;
    readonly joinColumnMetadatas: PropertyMetadataCollection<JoinColumnMetadata>;
    readonly joinTableMetadatas: PropertyMetadataCollection<JoinTableMetadata>;
    readonly indexMetadatas: PropertyMetadataCollection<IndexMetadata>;
    readonly entityListenerMetadatas: PropertyMetadataCollection<EntityListenerMetadata>;
    constructor(tableMetadatas?: TargetMetadataCollection<TableMetadata>, namingStrategyMetadatas?: TargetMetadataCollection<NamingStrategyMetadata>, eventSubscriberMetadatas?: TargetMetadataCollection<EventSubscriberMetadata>, compoundIndexMetadatas?: TargetMetadataCollection<CompoundIndexMetadata>, columnMetadatas?: PropertyMetadataCollection<ColumnMetadata>, relationMetadatas?: PropertyMetadataCollection<RelationMetadata>, joinColumnMetadatas?: PropertyMetadataCollection<JoinColumnMetadata>, joinTableMetadatas?: PropertyMetadataCollection<JoinTableMetadata>, indexMetadatas?: PropertyMetadataCollection<IndexMetadata>, entityListenerMetadatas?: PropertyMetadataCollection<EntityListenerMetadata>);
    /**
     * Creates a new copy of the MetadataStorage with same metadatas as in current metadata storage, but filtered
     * by classes.
     */
    mergeWithAbstract(allTableMetadatas: TargetMetadataCollection<TableMetadata>, tableMetadata: TableMetadata): {
        compoundIndexMetadatas: TargetMetadataCollection<CompoundIndexMetadata>;
        columnMetadatas: PropertyMetadataCollection<ColumnMetadata>;
        relationMetadatas: PropertyMetadataCollection<RelationMetadata>;
        joinColumnMetadatas: PropertyMetadataCollection<JoinColumnMetadata>;
        joinTableMetadatas: PropertyMetadataCollection<JoinTableMetadata>;
        indexMetadatas: PropertyMetadataCollection<IndexMetadata>;
        entityListenerMetadatas: PropertyMetadataCollection<EntityListenerMetadata>;
    };
}
