"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var PropertyMetadata_1 = require("./PropertyMetadata");
var RelationTypes_1 = require("./types/RelationTypes");
/**
 * This metadata interface contains all information about some document's relation.
 */
var RelationMetadata = (function (_super) {
    __extends(RelationMetadata, _super);
    // ---------------------------------------------------------------------
    // Constructor
    // ---------------------------------------------------------------------
    function RelationMetadata(args) {
        _super.call(this, args.target, args.propertyName);
        /**
         * Indicates if relation column value can be nullable or not.
         */
        this.isNullable = true;
        this.relationType = args.relationType;
        this._inverseSideProperty = args.inverseSideProperty;
        if (args.options.name)
            this._name = args.options.name;
        if (args.options.cascadeInsert)
            this.isCascadeInsert = args.options.cascadeInsert;
        if (args.options.cascadeUpdate)
            this.isCascadeUpdate = args.options.cascadeUpdate;
        if (args.options.cascadeRemove)
            this.isCascadeRemove = args.options.cascadeRemove;
        if (args.options.oldColumnName)
            this.oldColumnName = args.options.oldColumnName;
        if (args.options.nullable)
            this.isNullable = args.options.nullable;
        if (args.options.onDelete)
            this.onDelete = args.options.onDelete;
        if (!this._type)
            this._type = args.type;
        if (!this._name)
            this._name = args.propertyName;
    }
    Object.defineProperty(RelationMetadata.prototype, "name", {
        // ---------------------------------------------------------------------
        // Accessors
        // ---------------------------------------------------------------------
        get: function () {
            return this.namingStrategy ? this.namingStrategy.relationName(this._name) : this._name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RelationMetadata.prototype, "isOwning", {
        /**
         * Indicates if this side is an owner of this relation.
         */
        get: function () {
            return this.isManyToOne ||
                (this.isManyToMany && this.joinTable) ||
                (this.isOneToOne && this.joinColumn);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RelationMetadata.prototype, "type", {
        get: function () {
            return this._type();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RelationMetadata.prototype, "inverseSideProperty", {
        get: function () {
            return this.computeInverseSide(this._inverseSideProperty);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RelationMetadata.prototype, "inverseRelation", {
        get: function () {
            return this.relatedEntityMetadata.findRelationWithPropertyName(this.computeInverseSide(this._inverseSideProperty));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RelationMetadata.prototype, "isOneToOne", {
        get: function () {
            return this.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RelationMetadata.prototype, "isOneToMany", {
        get: function () {
            return this.relationType === RelationTypes_1.RelationTypes.ONE_TO_MANY;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RelationMetadata.prototype, "isManyToOne", {
        get: function () {
            return this.relationType === RelationTypes_1.RelationTypes.MANY_TO_ONE;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RelationMetadata.prototype, "isManyToMany", {
        get: function () {
            return this.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RelationMetadata.prototype, "hasInverseSide", {
        get: function () {
            return !!this.inverseRelation;
        },
        enumerable: true,
        configurable: true
    });
    // ---------------------------------------------------------------------
    // Private Methods
    // ---------------------------------------------------------------------
    RelationMetadata.prototype.computeInverseSide = function (inverseSide) {
        var ownerEntityPropertiesMap = this.relatedEntityMetadata.createPropertiesMap();
        if (typeof inverseSide === "function")
            return inverseSide(ownerEntityPropertiesMap);
        if (typeof inverseSide === "string")
            return inverseSide;
        // throw new Error("Cannot compute inverse side of the relation");
        return "";
    };
    return RelationMetadata;
}(PropertyMetadata_1.PropertyMetadata));
exports.RelationMetadata = RelationMetadata;
//# sourceMappingURL=RelationMetadata.js.map